import inspect

import requests
from simplejson import JSONDecodeError
import json

from src.phinix.exceptions import *


def validate_response(response: dict) -> bool:
    return response.get('success') or str(response.get('success')).lower() == 'true'


def get_token(mobile_number: str, password: str) -> str:
    func_name = inspect.currentframe().f_code.co_name

    payload = json.dumps({
        "mobile_number": mobile_number,
        "password": password
    })
    headers = {
        'Content-Type': 'application/json'
    }

    try:
        r = requests.post('https://api.phinix.ir/auth/login', headers=headers, data=payload)
    except Exception as e:
        raise RequestsExceptions(func_name, e)

    status_code = r.status_code

    if status_code == 200:
        try:
            resp = r.json()
        except JSONDecodeError as e:
            raise JsonDecodingError(func_name, e)

        if validate_response(resp):
            return resp.get('result').get('token')
        else:
            raise JsonDecodingError(func_name, r.text)
    else:
        raise StatusCodeError(func_name, status_code, r.text)


class Phinix:
    def __init__(self, token: str):
        self.base_url = "https://api.phinix.ir/v1/"
        self.session = requests.Session()
        self.session.headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {token}'
        }
        pass

    def order_book(self, symbol: str):
        func_name = inspect.currentframe().f_code.co_name
        try:
            r = self.session.get(self.base_url + f'depth?symbol={symbol}')
        except Exception as e:
            raise RequestsExceptions(func_name, e)

        status_code = r.status_code

        if status_code == 200:
            try:
                resp = r.json()
            except JSONDecodeError as e:
                raise JsonDecodingError(func_name, e)

            if validate_response(resp):
                return resp.get('result')
            else:
                raise JsonDecodingError(func_name, r.text)

        else:
            raise StatusCodeError(func_name, status_code, r.text)

    def all_recent_trades(self, symbol: str):
        func_name = inspect.currentframe().f_code.co_name
        try:
            r = self.session.get(self.base_url + f'trades?symbol={symbol}')
        except Exception as e:
            raise RequestsExceptions(func_name, e)

        status_code = r.status_code

        if status_code == 200:
            try:
                resp = r.json()
            except JSONDecodeError as e:
                raise JsonDecodingError(func_name, e)

            if validate_response(resp):
                return resp.get('result')
            else:
                raise JsonDecodingError(func_name, r.text)

        else:
            raise StatusCodeError(func_name, status_code, r.text)

    def all_balances(self):
        func_name = inspect.currentframe().f_code.co_name
        try:
            r = self.session.get(self.base_url + 'account/balances')
        except Exception as e:
            raise RequestsExceptions(func_name, e)

        status_code = r.status_code

        if status_code == 200:
            try:
                resp = r.json()
            except JSONDecodeError as e:
                raise JsonDecodingError(func_name, e)

            if validate_response(resp):
                return resp.get('result').get('balances')
            else:
                raise JsonDecodingError(func_name, r.text)

        else:
            raise StatusCodeError(func_name, status_code, r.text)

    def coin_balance(self, coin: str):
        coin = coin.upper()
        func_name = inspect.currentframe().f_code.co_name
        try:
            r = self.session.get(self.base_url + 'account/balances')
        except Exception as e:
            raise RequestsExceptions(func_name, e)

        status_code = r.status_code

        if status_code == 200:
            try:
                resp = r.json()
            except JSONDecodeError as e:
                raise JsonDecodingError(func_name, e)

            if validate_response(resp):
                return resp.get('result').get('balances').get(coin)
            else:
                raise JsonDecodingError(func_name, r.text)

        else:
            raise StatusCodeError(func_name, status_code, r.text)

    def coin_available_balance(self, coin: str):
        coin = coin.upper()
        func_name = inspect.currentframe().f_code.co_name
        try:
            r = self.session.get(self.base_url + 'account/balances')
        except Exception as e:
            raise RequestsExceptions(func_name, e)

        status_code = r.status_code

        if status_code == 200:
            try:
                resp = r.json()
            except JSONDecodeError as e:
                raise JsonDecodingError(func_name, e)

            if validate_response(resp):
                _ = resp.get('result').get('balances').get(coin)
                return float(_.get('value')) - float(_.get('locked'))
            else:
                raise JsonDecodingError(func_name, r.text)

        else:
            raise StatusCodeError(func_name, status_code, r.text)

    def create_order(self, price: str, quantity: str, side: str, symbol: str, type_: str, client_id: str = None):
        func_name = inspect.currentframe().f_code.co_name

        try:
            payload = json.dumps({
                "price": price,
                "quantity": quantity,
                "side": side.lower(),
                "symbol": symbol.upper(),
                "type": type_.lower(),
                "client_id": client_id,
            })
            r = self.session.post(self.base_url + 'account/orders', data=payload)
        except Exception as e:
            raise RequestsExceptions(func_name, e)

        status_code = r.status_code

        if status_code == 200:
            try:
                resp = r.json()
            except JSONDecodeError as e:
                raise JsonDecodingError(func_name, e)

            if validate_response(resp):
                return resp.get('result')
            else:
                raise JsonDecodingError(func_name, r.text)

        else:
            raise StatusCodeError(func_name, status_code, r.text)

    def cancel_order(self, client_id: str):
        func_name = inspect.currentframe().f_code.co_name

        try:
            payload = json.dumps({
                "clientOrderId": client_id
            })
            r = self.session.delete(self.base_url + 'account/orders', data=payload)
        except Exception as e:
            raise RequestsExceptions(func_name, e)

        status_code = r.status_code

        if status_code == 200:
            try:
                resp = r.json()
            except JSONDecodeError as e:
                raise JsonDecodingError(func_name, e)

            if validate_response(resp):
                return resp.get('result')
            else:
                raise JsonDecodingError(func_name, r.text)

        else:
            raise StatusCodeError(func_name, status_code, r.text)

    def open_orders(self, symbol: str):
        func_name = inspect.currentframe().f_code.co_name

        try:
            r = self.session.delete(self.base_url + f'account/openOrders?symbol={symbol.upper()}')
        except Exception as e:
            raise RequestsExceptions(func_name, e)

        status_code = r.status_code

        if status_code == 200:
            try:
                resp = r.json()
            except JSONDecodeError as e:
                raise JsonDecodingError(func_name, e)

            if validate_response(resp):
                return resp.get('result').get('orders')
            else:
                raise JsonDecodingError(func_name, r.text)

        else:
            raise StatusCodeError(func_name, status_code, r.text)

    def user_recent_trades(self, symbol: str = None, side: str = None, active: bool = None):
        func_name = inspect.currentframe().f_code.co_name

        params = {}

        if symbol:
            params.update({'symbol': symbol.upper()})
        if side:
            params.update({'side': side.lower()})
        if active:
            params.update({'active': active})

        try:
            r = self.session.get(self.base_url + f'account/trades', params=params)
        except Exception as e:
            raise RequestsExceptions(func_name, e)

        status_code = r.status_code

        if status_code == 200:
            try:
                resp = r.json()
            except JSONDecodeError as e:
                raise JsonDecodingError(func_name, e)

            if validate_response(resp):
                return resp.get('result').get('AccountLatestTrades')
            else:
                raise JsonDecodingError(func_name, r.text)

        else:
            raise StatusCodeError(func_name, status_code, r.text)



from typing import Union


class PhinixExceptions(Exception):
    def __init__(self, func_name: str, message: Union[str, Exception]):
        self.func_name = func_name
        self.message = str(message)
        super().__init__(self.message)

    def __str__(self):
        return f'"{self.func_name}" -> {self.message}'


class RequestsExceptions(PhinixExceptions):
    pass


class StatusCodeError(PhinixExceptions):
    def __init__(self, func_name: str, status_code: int, message: Union[str, Exception]):
        self.func_name = func_name
        self.status_code = status_code
        self.message = message
        super().__init__(func_name, message)

    def __str__(self):
        return f'{self.func_name} | {self.status_code} -> {self.message}'


class JsonDecodingError(PhinixExceptions):
    def __init__(self, func_name: str, message: Union[str, Exception]):
        self.func_name = func_name
        self.message = message
        super().__init__(func_name, message)

    def __str__(self):
        return f'{self.func_name} -> {self.message}'


class InvalidResponse(PhinixExceptions):
    def __init__(self, func_name: str, message: Union[str, Exception]):
        self.func_name = func_name
        self.message = message
        super().__init__(func_name, message)

    def __str__(self):
        return f'{self.func_name} -> {self.message}'



from distutils.core import setup

setup(
    name='phinix',
    packages=['phinix'],
    version='0.0.2',
    license='MIT',
    description='Phinix cryptocurrency exchange python sdk',
    author='amiwrpremium',
    author_email='amiwrpremium@gmail.com',
    url='https://github.com/amiwrpremium/phinix',
    keywords=['phinix', 'crypto', 'exchange', 'API', "SDK"],
    install_requires=[
        'requests',
        'simplejson',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
